{***********************************************************************
Unit PreviewPanel.pas.PAS v1.0
    (c) by Andreas Moser, amoser@amoser.de

    PreviewPanel is part of the gfx_library collection

    You may use this sourcecode for your freewareproducts.
    You may modify this source-code for your own use.
    You may recompile this source-code for your own use.

    All functions, procedures and classes may NOT be used in commercial
    products without the permission of the author. For parts of this library
    not written by me, you have to ask for permission by their
    respective authors.

    Disclaimer of warranty: "This software is supplied as is. The author
    disclaims all warranties, expressed or implied, including, without
    limitation, the warranties of merchantability and of fitness for any
    purpose. The author assumes no liability for damages, direct or
    consequential, which may result from the use of this software."

    All brand and product names are marks or registered marks of their
    respective companies.

    Please report bugs to:
    Andreas Moser  amoser@amoser.de

********************************************************************************}
unit PreviewPanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,StdCtrls, gfx_RotResize,gfx_colors,gfx_colorpalette,gfx_files,vcl_tools;

type TOrientation=(ppVertical,ppHorizontal);

const ZoomVal=2;
var
ZoomFactors: Array [0..9] of Integer=(25,33,50,75,100,200,400,600,1000,1600);
type
  TPreviewPanel = class(TPanel)
  private
    FOnOrgPosChanged:TNotifyEvent;
    fx,fy:LongInt;
    fPosX,fPosY:LongInt;
    fZoomFactor:Integer;
    fSourceBitmap:TBitmap;
    fZoomedBitmap:TBitmap;
    fOriginalImage:TImage;
    fPreviewImage:TImage;
    fZoomPlusBtn:TButton;
    fZoomMinusBtn:TButton;
    fBevel1,fBevel2:TBevel;
    fOrientation:TOrientation;
    fChanged:Boolean;
    fUpdatePrev:Boolean;
    Procedure SetOrientation(Value:TOrientation);
    Procedure SetSourceBitmap(Value:TBitmap);
    procedure BitmapChanged(Sender:TObject);
    procedure ButtonPlusClick(Sender:TObject);
    procedure ButtonMinusClick(Sender:TObject);
    procedure OriginalMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure OriginalMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure OriginalMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    { Private-Deklarationen }
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
    procedure UpdatePreview;
    procedure LoadSourceBitmap(Filename:String);
    property SourceBitmap:TBitmap read fSourceBitmap write SetSourceBitmap;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property OriginalImage:TImage read fOriginalImage;
    property PreviewImage:TImage read fPreviewImage;
    property Orientation:TOrientation read fOrientation write SetOrientation;
    property OnOrgPosChanged:TNotifyEvent read fOnOrgPosChanged write FonOrgPosChanged;
    { Published-Deklarationen }
  end;

procedure Register;

implementation

constructor TPreviewPanel.Create(AOwner: TComponent);
//---------------------------------------------------------------
//  Purpose:Constructor
//---------------------------------------------------------------

begin
  inherited Create(AOwner);
  FOrientation:=ppVertical;
  Self.Width:=120;
  Self.Height:=250;
  Self.Caption:='';
  fBevel1:=TBevel.Create(self);
  fBevel2:=TBevel.Create(self);
  fBevel1.Shape:=bsFrame;
  fBevel2.Shape:=bsFrame;
  fBevel1.Parent:=self;
  fBevel2.Parent:=self;
  FZoomedBitmap:=TBitmap.Create;
  fZoomedBitmap.PixelFormat:=pf24Bit;
  fSourceBitmap:=TBitmap.Create;
  fSourceBitmap.OnChange:=BitmapChanged;
  FOriginalImage:=TImage.Create(Self);
  with FOriginalImage do
  begin
    Parent:=self;
    Center:=true;
    OnMouseMove:=OriginalMove;
    OnMouseUp:=OriginalMouseUp;
    OnMouseDown:=OriginalMouseDown;
  end;
  FPreviewImage:=TImage.Create(Self);
  FPreviewImage.Parent:=self;
  FPreviewImage.Center:=true;
  FZoomPlusBtn:=TButton.Create(self);
  FZoomPlusBtn.Parent:=self;
  FZoomPlusBtn.Caption:='+';
  FZoomPlusBtn.Width:=15;
  FZoomPlusBtn.Height:=15;
  FZoomPlusBtn.OnClick:=ButtonPlusClick;
  FZoomMinusBtn:=TButton.Create(self);
  FZoomMinusBtn.Parent:=self;
  FZoomMinusBtn.Caption:='-';
  FZoomMinusBtn.Width:=FZoomPlusBtn.Width;
  FZoomMinusBtn.Height:=FZoomPlusBtn.Height;
  FZoomMinusBtn.OnClick:=ButtonMinusClick;
  fx:=0;fy:=0;fPosX:=0;fPosY:=0;fChanged:=false;
  fZoomFactor:=4;
  Caption:='';
  if GetDeviceCaps(GetDC(0),BITSPIXEL)=8 then
  begin
//    ApplyPaletteToGraphic(LoadOptimizedPalette,fOriginalImage.Picture.Bitmap,true,pf8Bit);
//    ApplyPaletteToGraphic(LoadOptimizedPalette,fPreviewImage.Picture.Bitmap,true,pf8Bit);
  end else
  begin
    fOriginalImage.Picture.Bitmap.PixelFormat:=pf24Bit;
    fPreviewImage.Picture.Bitmap.PixelFormat:=pf24Bit;
  end;

  SetOrientation(FOrientation);
  fUpdatePrev:=True;
end;

destructor TPreviewPanel.Destroy;
//---------------------------------------------------------------
//  Purpose: Destructor
//---------------------------------------------------------------
begin
  fOriginalImage.Free;
  fPreviewImage.Free;
  fSourceBitmap.Free;
  fZoomedBitmap.Free;
  inherited Destroy;
end;

Procedure TPreviewPanel.SetOrientation(Value:TOrientation);
//---------------------------------------------------------------
//  Purpose: Set the orientation of the panel
//---------------------------------------------------------------
begin
  FOrientation:=Value;
  case FOrientation of
    ppVertical:
    begin
      FOriginalImage.Width:=(Width -10);
      FOriginalImage.Left:=5;
      FOriginalImage.Height:=Height div 2 -(FZoomMinusBtn.Height div 2 +10);
      FOriginalImage.Top:=5;
      FZoomMinusBtn.Top:=FOriginalImage.Height+FOriginalImage.Top+5;
      FZoomPlusBtn.Top:=FOriginalImage.Height+FOriginalImage.Top+5;
      FZoomMinusBtn.Left:=Width div 2 -(2+FZoomMinusBtn.Width);
      FZoomPlusBtn.Left:=Width div 2 +2;
      FPreviewImage.Width:=FOriginalImage.Width;
      FPreviewImage.Left:=5;
      FPreviewImage.Height:=FOriginalImage.Height;
      FPreviewImage.Top:=FZoomMinusBtn.Height+FZoomMinusBtn.Top+5;
    end;
    ppHorizontal:
    begin
      FOriginalImage.Width:=Width div 2 -(FZoomMinusBtn.Width +10);
      FOriginalImage.Left:=5;
      FOriginalImage.Height:=(Height -10);;
      FOriginalImage.Top:=5;
      FZoomMinusBtn.Top:=Height div 2 -(FZoomMinusBtn.Height div 2);
      FZoomMinusBtn.Left:=Width div 2 -(2+FZoomMinusBtn.Width);
      FZoomPlusBtn.Top:=FZoomMinusBtn.Top;
      FZoomPlusBtn.Left:=Width div 2 +2;
      FPreviewImage.Width:=FOriginalImage.Width;
      FPreviewImage.Height:=FOriginalImage.Height;
      FPreviewImage.Top:=5;
      FPreviewImage.Left:=FZoomPlusBtn.Left+FZoomPlusBtn.Width+2;
    end;
  end;
  with fBevel1 do
  begin
     Left:=fOriginalImage.Left-2;
     Top:=fOriginalImage.Top-2;
     Width:=fOriginalImage.Width+4;
     Height:=fOriginalImage.Height+4;
  end;
  with fBevel2 do
  begin
     Left:=fPreviewImage.Left-2;
     Top:=fPreviewImage.Top-2;
     Width:=fPreviewImage.Width+4;
     Height:=fPreviewImage.Height+4;
  end;
end;

procedure TPreviewPanel.LoadSourceBitmap(Filename:String);
//---------------------------------------------------------------
//  Purpose: Load the bitmap to be processed
//---------------------------------------------------------------
begin
  fSourceBitmap.OnChange:=nil;
  LoadAnyImageToBMP(FSourceBitmap,FileName,false,0,0);

      if GetDeviceCaps(GetDC(0),BITSPIXEL)=8 then
    begin
      fOriginalImage.Picture.Bitmap.PixelFormat:=pf8Bit;
      fOriginalImage.Picture.Bitmap.Palette:=
      CreateOptimizedPaletteForSingleBitmap(FSourceBitmap,8);
      fOriginalImage.Picture.Bitmap.IgnorePalette:=false;
    end;

  BitmapChanged(Self);
  fSourceBitmap.OnChange:=BitmapChanged;
end;

Procedure TPreviewPanel.SetSourceBitmap(Value:TBitmap);
//---------------------------------------------------------------
//  Purpose: Set the source bitmap
//---------------------------------------------------------------
begin
  if Value <>fSourceBitmap then if
  Assigned(Value) then
  begin
    fSourceBitmap:=Value;
    fSourceBitmap.OnChange:=BitmapChanged;
    fPosX:=0;fPosY:=0;
    if GetDeviceCaps(GetDC(0),BITSPIXEL)=8 then
    begin
      fOriginalImage.Picture.Bitmap.PixelFormat:=pf8Bit;
      fOriginalImage.Picture.Bitmap.Palette:=
      CreateOptimizedPaletteForSingleBitmap(FSourceBitmap,8);
      fOriginalImage.Picture.Bitmap.IgnorePalette:=false;
//      ApplyPaletteToDC(GetDC(self.Handle),fOriginalImage.Picture.Bitmap.Palette);
    end;
    BitmapChanged(self);

  end;
end;

procedure TPreviewPanel.ButtonPlusClick(Sender:TObject);
//---------------------------------------------------------------
//  Purpose: Button '+' event
//---------------------------------------------------------------
begin
  if fZoomFactor <9 then
  begin
    Inc(fZoomFactor,1);
    BitmapChanged(self);
   if Assigned(FOnOrgPosChanged) then FOnOrgPosChanged(Self);
   fChanged:=false;
  end;
end;

procedure TPreviewPanel.ButtonMinusClick(Sender:TObject);
//---------------------------------------------------------------
//  Purpose: Button '-' event
//---------------------------------------------------------------
begin
  if fZoomFactor >0 then
  begin
    Dec(fZoomFactor,1);
   fPosX:=0;
   fPosY:=0;
   BitmapChanged(self);
   if Assigned(FOnOrgPosChanged) then FOnOrgPosChanged(Self);
   fChanged:=false;
  end;
end;

procedure TPreviewPanel.OriginalMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
//---------------------------------------------------------------
//  Purpose: Mouse up event
//---------------------------------------------------------------
begin
 if fChanged=true then begin
   if Assigned(FOnOrgPosChanged) then FOnOrgPosChanged(Self);
   fChanged:=false;
 end;
end;

procedure TPreviewPanel.OriginalMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
//---------------------------------------------------------------
//  Purpose: Mouse down event
//---------------------------------------------------------------
begin
   fx:=x;
   fy:=y;
end;

procedure TPreviewPanel.OriginalMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
//---------------------------------------------------------------
//  Purpose: Mouse move event
//---------------------------------------------------------------
var dx,dy:Integer;
begin
 if (ssLeft in Shift)
 then begin
//   zF:=Round(ZoomFactors[fZoomFactor]);
   dx:=(fx-x);
   dy:=(fy-y);
   fPosX:=fPosX+dx;
   if fposX<=0 then fposX:=0 else
   if fposX>=(fSourceBitmap.Width*(ZoomFactors[fZoomFactor]/100))-fOriginalImage.Picture.Bitmap.Width then
   fPosX:=Round(fSourceBitmap.Width*(ZoomFactors[fZoomFactor]/100))-fOriginalImage.Picture.Bitmap.Width;

   fPosY:=fPosY+dY;
   if fposY<0 then fposY:=0 else
   if fposY>=(fSourceBitmap.Height*(ZoomFactors[fZoomFactor]/100))-fOriginalImage.Picture.Bitmap.Height then
   fPosY:=Round(fSourceBitmap.Height*(ZoomFactors[fZoomFactor]/100))-fOriginalImage.Picture.Bitmap.Height;

   fUpdatePrev:=False;
   BitmapChanged(Self);
   fUpdatePrev:=true;
   fx:=x;
   fy:=y;
   fChanged:=True;
 end;
end;


procedure TPreviewPanel.UpdatePreview;
//---------------------------------------------------------------
//  Purpose: Update preview
//---------------------------------------------------------------
begin
  fPreviewImage.Canvas.Brush.Color:=Color;
  fPreViewImage.Canvas.FillRect(fPreViewImage.ClientRect);
  fPreviewImage.Width:=fOriginalImage.Width;
  fPreviewImage.Height:=fOriginalImage.Height;
  fPreviewImage.Picture.Bitmap.width:=fOriginalImage.Picture.Bitmap.Width;
  fPreviewImage.Picture.Bitmap.Height:=fOriginalImage.Picture.Bitmap.Height;
end;

procedure TPreviewPanel.BitmapChanged(Sender:TObject);
//---------------------------------------------------------------
//  Purpose: Bitmap changed event
//---------------------------------------------------------------
var zW,zH,zF:Integer;
begin
  if Assigned(fSourceBitmap) then
  begin
    zF:=ZoomFactors[fZoomFactor];

    fOriginalImage.Picture.Bitmap.Width:=fOriginalImage.Width;
    fOriginalImage.Picture.Bitmap.Height:=fOriginalImage.Height;

    //  if zF>=100 then
    if (zF>=100) and (fSOurceBitmap.Width >= fOriginalImage.Width) then   // changed, Thanks to Francisco Camilo Barreto
    begin
      zW:=Round(fOriginalImage.Picture.Bitmap.Width/(zF/100));
      zH:=Round(fOriginalImage.Picture.Bitmap.Height/(zF/100));
      fZoomedBitmap.Width:=zW;
      fZoomedBitmap.Height:=zH;

      { For the following, thanks to Francisco Camilo Barreto }

      if fPosX >= fSourceBitmap.Width-zW then
        fPosX := fSourceBitmap.Width-zW;
      if fPosY >= fSourceBitmap.Height-zH then
        fPosY := fSourceBitmap.Height-zH;

      {end of add}

      fZoomedBitmap.Canvas.CopyRect(Rect(0,0,zW,zH),fSourceBitmap.Canvas,Rect(fPosX,fPosY,fPosX+zW,fPosY+zH));
      fOriginalImage.Picture.Bitmap.Canvas.StretchDraw(Rect(0,0,fOriginalImage.Picture.Bitmap.Width,fOriginalImage.Picture.Bitmap.Height),FZoomedBitmap);
    end else
    if (zF>=100) then
    begin
      zW:=Round(fOriginalImage.Picture.Bitmap.Width/(zF/100));
      zH:=Round(fOriginalImage.Picture.Bitmap.Height/(zF/100));
      fZoomedBitmap.Width:=zW;
      fZoomedBitmap.Height:=zH;

      { For the following, thanks to Francisco Camilo Barreto }

      if fPosX >= fSourceBitmap.Width-zW then
        fPosX := fSourceBitmap.Width-zW;
      if fPosY >= fSourceBitmap.Height-zH then
        fPosY := fSourceBitmap.Height-zH;

      {end of add}

      fZoomedBitmap.Canvas.CopyRect(Rect(0,0,zW,zH),fSourceBitmap.Canvas,Rect(fPosX,fPosY,fPosX+zW,fPosY+zH));
      fOriginalImage.Picture.Bitmap.Canvas.StretchDraw(Rect(0,0,fOriginalImage.Picture.Bitmap.Width,fOriginalImage.Picture.Bitmap.Height),FZoomedBitmap);
    end else
    if zF<100 then
    begin
      zW:=Round(fSourceBitmap.Width*(zF/100));
      zH:=Round(fSourceBitmap.Height*(zF/100));
      fZoomedBitmap.Width:=zW;
      fZoomedBitmap.Height:=zH;
      if zW<fOriginalImage.Width then begin
        fOriginalImage.Picture.Bitmap.Width:=zW;
        if zW<=fOriginalImage.Width then fPosX:=0;
      end;
      if zH<fOriginalImage.Height then begin
        fOriginalImage.Picture.Bitmap.Height:=zH;
        if zH<=fOriginalImage.Height then fPosY:=0;
      end;
      fZoomedBitmap.Canvas.StretchDraw(Rect(0,0,zW,zH),FSourceBitmap);
      fOriginalImage.Picture.Bitmap.Canvas.CopyRect(Rect(0,0,fOriginalImage.Picture.Bitmap.Width,fOriginalImage.Picture.Bitmap.Height),
      fZoomedBitmap.Canvas,Rect(fPosX,fPosY,fPosX+fOriginalImage.Picture.Bitmap.Width,fposY+fOriginalImage.Picture.Bitmap.Height))
    end;

    fOriginalImage.Repaint;

    if fUpdatePrev then UpdatePreview;
  end;
end;

procedure Register;
//---------------------------------------------------------------
//  Purpose: Register
//---------------------------------------------------------------
begin
  RegisterComponents('AM FX Pro', [TPreviewPanel]);
end;

end.
