unit gfx_DIBProcessing;
{***********************************************************************

Unit gfx_DIBProcessing.PAS v1.5 0801
    (c) by Andreas Moser, amoser@amoser.de,

    Delphi version : Delphi 4, 6 (due to user responses it seem to work with Delphi 5)

    gfx_DIBProcessing is part of the gfx_library collection

    You may use this sourcecode for your freewareproducts.
    You may modify this source-code for your own use.
    You may recompile this source-code for your own use.

    All functions, procedures and classes may NOT be used in commercial
    products without the permission of the author. For parts of this library
    not written by me, you have to ask for permission by their
    respective authors.

    Disclaimer of warranty: "This software is supplied as is. The author
    disclaims all warranties, expressed or implied, including, without
    limitation, the warranties of merchantability and of fitness for any
    purpose. The author assumes no liability for damages, direct or
    consequential, which may result from the use of this software."

    All brand and product names are marks or registered marks of their
    respective companies.

    Please report bugs to:
    Andreas Moser  amoser@amoser.de

    for more information about image processing look at this book:

********************************************************************************}

interface

uses gfx_basedef,
     gfx_dib;

procedure DIBEffectStaticFilter(BmpIntfc:TBMPInterface1;StaticFilterType:TStaticFilterType;Diameter:Integer;ColorSpace:TColorSpace;C1,C2,C3,C4:Boolean;const EffectCallBack:TEffectCallBack);stdcall;
procedure DIBEffectLinearFilter(BmpIntfc:TBMPInterface1;Filter:TGraphicFilter;ColorSpace:TColorSpace;C1,C2,C3,C4:Boolean;const EffectCallBack:TEffectCallBack);stdcall;
procedure DIBEffectArithmetic(BMPIntfc:TBMPInterface2;Arithmetic:TGraphicArithmetic;C1,C2,C3:Boolean;Multiplicator,MaxBlendValue:Integer;const EffectCallBack:TEffectCallBack);stdcall;

procedure DIBEffect1Val(BMPIntfc:TBMPInterface1;EffectType:TFXType;C1,C2,C3:Boolean;Value:Variant;Switch:Boolean;const EffectCallBack:TEffectCallBack);stdCall;
procedure DIBEffect2Val(BMPIntfc:TBMPInterface1;EffectType:TFXType;C1,C2,C3:Boolean;Value1,Value2:Variant;Switch:Boolean;const EffectCallBack:TEffectCallBack);stdCall;
procedure DIBEffect3Val(BMPIntfc:TBMPInterface1;EffectType:TFXType;C1,C2,C3:Boolean;Value1,Value2,Value3:Variant;Switch:Boolean;const EffectCallBack:TEffectCallBack);stdCall;

// Helper functions and procedures
function  ComputePixel(x, y:double;var x1,y1:double):Integer;

implementation



// -----------------------------------------------------------------------------
//                       *******************
//                       DIBEffecStaticFilter
//                       *******************
// -----------------------------------------------------------------------------
//
//     Bassic parameters:
//               BmpIntfc: TBMPInterface2 (Two source bitmaps,one destination)
//               StaticFilterType: sfMedian,sfMin,sfMax
//               Diameter:         value [3,5,7] ("radius")
//               Colorspace:       currently not used
//               C1,C2,C3: Color planes to be used (C1,C2,C3)
// -----------------------------------------------------------------------------
procedure DIBEffectStaticFilter(BmpIntfc:TBMPInterface1;StaticFilterType:TStaticFilterType;Diameter:Integer;ColorSpace:TColorSpace;C1,C2,C3,C4:Boolean;const EffectCallBack:TEffectCallBack);stdcall;
var
  row,col           :Integer;
  k,i               :Integer;
  mx,my           :Integer;
  HorzLine,VertLine :Integer;
  MaxCol,MaxRow     :Integer;
  MinCol,MinRow     :Integer;
  TargetRow         :pRGBArray;
  lLightness        :TEffectMatrix;//array[0..6,0..6] of Integer;
  BorderDiffX,BorderDiffY :Integer;
  HSL1,HSL2         :THSLTriple;
  HSV1,HSV2         :THSVTriple;
  CMYK1,CMYK2         :TCMYKQuad;
begin
 if Diameter in [3,5,7] then
 begin
    Diameter:=Diameter DIV 2;
    for k:=0 to MaxEffectMatrixVector-1 do
    for i:=0 to MaxEffectMatrixVector-1 do lLightness[k,i]:=0;

    MinRow:=-Diameter;
    MaxRow:=Diameter;
    MinCol:=-Diameter;
    Maxcol:=Diameter;

      for Row := 0 to BmpIntfc.Height - 1 do
      begin

        if Assigned(EffectCallBack) then EffectCallBack(0,100,Round((Row/BmpIntfc.Height)*100));

        TargetRow := BMPIntfc.DestScanline[Row];
        for Col := 0 to BmpIntfc.Width - 1 do
        begin

          mx:=0;my:=0;

          // fill the array with the lightness of every pixel
          for VertLine:=MinRow to MaxRow do
          begin
          {To handle the borders of a Bitmap i use the variable BorderDiff.
          In case of a borderpixel the pixelarray will be mirrored to the "illegal" coordinates}
          if ((Row< Diameter) AND (Vertline<0)) OR((Row> BmpIntfc.Height-1-Diameter) AND (Vertline>0))
          then BorderDiffY:=-1 ELSE BorderDiffY:=1;
            for HorzLine:=MinCol to MaxCol do
            begin
              if ((Col< Diameter) AND (Horzline<0)) OR ((Col> BmpIntfc.Width-1-Diameter) AND (Horzline>0))
              then BorderDiffX:=-1 ELSE
              BorderDiffX:=1;
              lLightness[Vertline+3,Horzline+3]:=RGBLightness(BMPIntfc.SourceRGBTriple[Row+(VertLine*BorderDiffY),Col+(HorzLine*BorderDiffX)]);
            end;
          end;

          GetStatOfMatrix(lLightness,StaticFilterType,Diameter,my,mx);

          if ((Row< Diameter) AND (my<0)) OR((Row> BmpIntfc.Height-1-Diameter) AND (mY>0))
          then BorderDiffY:=-1 ELSE BorderDiffY:=1;
          if ((Col< Diameter) AND (mx<0)) OR ((Col> BmpIntfc.Width-1-Diameter) AND (mx>0))
          then BorderDiffX:=-1 ELSE
          BorderDiffX:=1;

          case ColorSpace of
          csRGB:
            begin
              if C1 then TargetRow[Col].rgbtRed  :=BMPIntfc.SourceRGBTriple[Row+(my*BorderDiffY),Col+(mx*BorderDiffX)].rgbtRed;
              if C2 then TargetRow[Col].rgbtGreen  :=BMPIntfc.SourceRGBTriple[Row+(my*BorderDiffY),Col+(mx*BorderDiffX)].rgbtGreen;
              if C3 then TargetRow[Col].rgbtBlue  :=BMPIntfc.SourceRGBTriple[Row+(my*BorderDiffY),Col+(mx*BorderDiffX)].rgbtBlue;
            end;
          csHSV:
            begin
                RGBtoHSV(BMPIntfc.SourceRGBTriple[Row+(my*BorderDiffY),Col+(mx*BorderDiffX)],HSV1);
                RGBtoHSV(BMPIntfc.SourceRGBTriple[Row,Col],HSV2);
                if not C1 then HSV1.hsvHue:=HSV2.hsvHue;
                if not C2 then HSV1.hsvSaturation:=HSV2.hsvSaturation;
                if not C3 then HSV1.hsvValue :=HSV2.hsvValue;
                HSVToRGB(HSV1,TargetRow[Col]);
            end;
          csHSL:
            begin
                RGBtoHSL(BMPIntfc.SourceRGBTriple[Row+(my*BorderDiffY),Col+(mx*BorderDiffX)],HSL1);
                RGBtoHSL(BMPIntfc.SourceRGBTriple[Row,Col],HSL2);
                if not C1 then HSL1.hslHue:=HSL2.hslHue;
                if not C2 then HSL1.hslSaturation:=HSL2.hslSaturation;
                if not C3 then HSL1.hslLightness :=HSL2.hslLightness;
                HSLToRGB(HSL1,TargetRow[Col]);
            end;
          csCMYK:
            begin
                RGBtoCMYK(BMPIntfc.SourceRGBTriple[Row+(my*BorderDiffY),Col+(mx*BorderDiffX)],CMYK1);
                RGBtoCMYK(BMPIntfc.SourceRGBTriple[Row,Col],CMYK2);
                if not C1 then CMYK1.cmykCyan:=CMYK2.cmykCyan;
                if not C2 then CMYK1.cmykMagenta:=CMYK2.cmykMagenta;
                if not C3 then CMYK1.cmykYellow :=CMYK2.cmykYellow;
                if not C4 then CMYK1.cmykK :=CMYK2.cmykK;
                CMYKToRGB(CMYK1,TargetRow[Col]);
            end;
          end;
        end;
      end;
 end;
end;


// -----------------------------------------------------------------------------
//                       *******************
//                       DIBEffectLinearFilter
//                       *******************
// -----------------------------------------------------------------------------
//
//     Bassic parameters:
//               BmpIntfc: TBMPInterface2 (Two source bitmaps,one destination)
//               Filter:           Filter to be applied
//               Colorspace:       Colorspace to be used (RGB,HSV, HSL)
//               C1,C2,C3: Color planes to be used (C1,C2,C3)
// -----------------------------------------------------------------------------
procedure DIBEffectLinearFilter(BmpIntfc:TBMPInterface1;Filter:TGraphicFilter;ColorSpace:TColorSpace;C1,C2,C3,C4:Boolean;const EffectCallBack:TEffectCallBack);stdcall;
var
  row,col                 :Integer;
  i,mxCount               :Integer;
  HorzLine,VertLine       :Integer;
  MaxCol,MaxRow           :Integer;
  MinCol,MinRow           :Integer;
  TargetRow               :pRGBArray;
  Val1,Val2,Val3,Val4     :array[0..6] of Integer;
  Val1Sum,Val2Sum,Val3Sum,
  Val4Sum                 :Integer;
  h1,s1,vl1               :Integer;
  hsv                     :THSVTriple;
  hsl                     :THSLTriple;
  cmyk                    :TCMYKQuad;
  SourceRows              :PPRows;
  BorderDiffX,BorderDiffY :Integer;
  Intensity,Intensity2    :Integer;
begin

    mxCount:=1;
    CASE Filter.MatrixSize of
      mx3:mxCount:=1;
      mx5:mxCount:=2;
      mx7:mxCount:=3;
    end;

    for Row := 0 to BmpIntfc.Height - 1 do
    begin

      if Assigned(EffectCallBack) then EffectCallBack(0,100,Round((Row/BmpIntfc.Height)*100));

      TargetRow:=BMPIntfc.DestScanline[Row];
      for Col := 0 to BMPIntfc.Width - 1 do
      begin
        Val1Sum:=0;
        Val2Sum:=0;
        Val3Sum:=0;
        Val4Sum:=0;
        MinRow:=-mxCount;
        MaxRow:=mxCount;
        MinCol:=-mxCount;
        Maxcol:=mxCount;
        for i:= MinRow to MaxRow do
        begin
          if C1 then Val1[i+3]:=0;
          if C2 then Val2[i+3]:=0;
          if C3 then Val3[i+3]:=0;
          if C4 then Val4[i+3]:=0;
        end;
        for VertLine:=MinRow to MaxRow do
        begin
          {to handle the borders of a Bitmap i use the variable BorderDiff.
           In case of a borderpixel the pixelarray will be mirrored to the "illegal" coordinates}
          if ((Row< mxCount) AND (Vertline<0)) OR((Row> BmpIntfc.Height-1-mxCount) AND (Vertline>0))
          then BorderDiffY:=-1 ELSE BorderDiffY:=1;
          for HorzLine:=MinCol to MaxCol do
          begin
            if ((Col< mxCount) AND (Horzline<0)) OR ((Col> BmpIntfc.Width-1-mxCount) AND (Horzline>0))
            then BorderDiffX:=-1 ELSE
            BorderDiffX:=1;
            CASE ColorSpace of
            csRGB:begin
                    if C1 then Val1[VertLine+3]:=Val1[VertLine+3]+(BMPIntfc.SourceRGBTriple[Row+VertLine*BorderDiffY,Col+HorzLine*BorderDiffX].rgbtRed*Filter.Matrix[HorzLine+3,VertLine+3]);
                    if C2 then Val2[VertLine+3]:=Val2[VertLine+3]+(BMPIntfc.SourceRGBTriple[Row+VertLine*BorderDiffY,Col+HorzLine*BorderDiffX].rgbtGreen*Filter.Matrix[HorzLine+3,VertLine+3]);
                    if C3 then Val3[Vertline+3]:=Val3[VertLine+3]+(BMPIntfc.SourceRGBTriple[Row+VertLine*BorderDiffY,Col+HorzLine*BorderDiffX].rgbtBlue*Filter.Matrix[HorzLine+3,VertLine+3]);
                  end;
            csHSV,csHSL:begin
                    case ColorSpace of
                    csHSV:begin
                            RGBtoHSV(BMPIntfc.SourceRGBTriple[Row+VertLine*BorderDiffY,Col+HorzLine*BorderDiffX],hsv);
                            if C1 then Val1[VertLine+3]:=Val1[VertLine+3]+(hsv.hsvHue*Filter.Matrix[VertLine+3,HorzLine+3])
                            ELSE Val1[VertLine+3]:=hsv.hsvHue;
                            if C2 then Val2[VertLine+3]:=Val2[VertLine+3]+(hsv.hsvSaturation*Filter.Matrix[VertLine+3,HorzLine+3])
                            ELSE Val2[VertLine+3]:=hsv.hsvSaturation;
                            if C3 then Val3[VertLine+3]:=Val3[VertLine+3]+(hsv.hsvValue*Filter.Matrix[VertLine+3,HorzLine+3])
                            ELSE Val3[VertLine+3]:=hsv.hsvValue;
                          end;
                    csHSL:begin
                            RGBtoHSL(BMPIntfc.SourceRGBTriple[Row+VertLine*BorderDiffY,Col+HorzLine*BorderDiffX],hsl);
                            if C1 then Val1[VertLine+3]:=Val1[VertLine+3]+(hsl.hslHue*Filter.Matrix[VertLine+3,HorzLine+3])
                            ELSE Val1[VertLine+3]:=hsl.hslHue;
                            if C2 then Val2[VertLine+3]:=Val2[VertLine+3]+(hsl.hslSaturation*Filter.Matrix[VertLine+3,HorzLine+3])
                            ELSE Val2[VertLine+3]:=hsl.hslSaturation;
                            if C3 then Val3[VertLine+3]:=Val3[VertLine+3]+(hsl.hslLightness*Filter.Matrix[VertLine+3,HorzLine+3])
                            ELSE Val3[VertLine+3]:=hsl.hslLightness;
                          end;
                    end;
                  end;
            csCMYK:
                  begin
                    RGBtoCMYK(BMPIntfc.SourceRGBTriple[Row+VertLine*BorderDiffY,Col+HorzLine*BorderDiffX],cmyk);
                    if C1 then Val1[VertLine+3]:=Val1[VertLine+3]+(cmyk.cmykCyan*Filter.Matrix[VertLine+3,HorzLine+3])
                    ELSE Val1[VertLine+3]:=cmyk.cmykCyan;
                    if C2 then Val2[VertLine+3]:=Val2[VertLine+3]+(cmyk.cmykMagenta*Filter.Matrix[VertLine+3,HorzLine+3])
                    ELSE Val2[VertLine+3]:=cmyk.cmykMagenta;
                    if C3 then Val3[VertLine+3]:=Val3[VertLine+3]+(cmyk.cmykYellow*Filter.Matrix[VertLine+3,HorzLine+3])
                    ELSE Val3[VertLine+3]:=cmyk.cmykYellow;
                    if C4 then Val4[VertLine+3]:=Val4[VertLine+3]+(cmyk.cmykK*Filter.Matrix[VertLine+3,HorzLine+3])
                    ELSE Val4[VertLine+3]:=cmyk.cmykK;
                  end;
            end;
          end;
        end;
        for i:=MinRow to MaxRow do
        begin
            if C1 then Val1Sum:=Val1Sum+Val1[i+3] else Val1Sum:=Val1[3];
            if C2 then Val2Sum:=Val2Sum+Val2[i+3] else Val2Sum:=Val2[3];
            if C3 then Val3Sum:=Val3Sum+Val3[i+3] else Val3Sum:=Val3[3];
            if C4 then Val4Sum:=Val4Sum+Val4[i+3] else Val4Sum:=Val4[3];
        end;
        if Filter.Divisor= 0 then Filter.Divisor:=1;
        CASE ColorSpace of
        csRGB:begin
                if C1 then TargetRow[Col].rgbtRed   := TrimReal(0,255,(Val1Sum*(1/Filter.Divisor))+Filter.Bias)
                ELSE TargetRow[Col].rgbtRed:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed;
                if C2 then TargetRow[Col].rgbtGreen :=TrimReal(0,255,(Val2Sum*(1/Filter.Divisor))+Filter.Bias)
                ELSE TargetRow[Col].rgbtGreen:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen;
                if C3 then TargetRow[Col].rgbtBlue  :=TrimReal(0,255,(Val3Sum*(1/Filter.Divisor))+Filter.Bias)
                ELSE TargetRow[Col].rgbtBlue:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue;
              end;
        csHSV:begin
                HSV.hsvHue:=   LoopReal(0,359,((1/Filter.Divisor)*Val1Sum)+Filter.Bias);
                HSV.hsvSaturation:=   TrimReal(0,255,((1/Filter.Divisor)*Val2Sum)+Filter.Bias);
                HSV.hsvValue:=   TrimReal(0,255,((1/Filter.Divisor)*Val3Sum)+Filter.Bias);
                HSVToRGB(HSV,TargetRow[Col]);
              end;
        csHSL:begin
                HSL.hslHue:=   LoopReal(0,359,((1/Filter.Divisor)*Val1Sum)+Filter.Bias);
                HSL.hslSaturation:=   TrimReal(0,255,((1/Filter.Divisor)*Val2Sum)+Filter.Bias);
                HSL.hslLightness:=  TrimReal(0,255,((1/Filter.Divisor)*Val3Sum)+Filter.Bias);
                HSLToRGB(HSL,TargetRow[Col]);
              end;
        csCMYK:begin
                cmyk.cmykCyan:=      TrimReal(0,255,((1/Filter.Divisor)*Val1Sum)+Filter.Bias);
                cmyk.cmykMagenta:=   TrimReal(0,255,((1/Filter.Divisor)*Val2Sum)+Filter.Bias);
                cmyk.cmykYellow:=    TrimReal(0,255,((1/Filter.Divisor)*Val3Sum)+Filter.Bias);
                cmyk.cmykK:=         TrimReal(0,255,((1/Filter.Divisor)*Val4Sum)+Filter.Bias);
                CMYKtoRGB(cmyk,TargetRow[Col]);
              end;
        end;
      end;
    end;
end;

// -----------------------------------------------------------------------------
//                       *******************
//                       DIBEffectArithmetic
//                       *******************
// -----------------------------------------------------------------------------
//
//     Bassic parameters:
//               BmpIntfc: TBMPInterface2 (Two source bitmaps,one destination)
//               C1,C2,C3: Color planes to be used (R,G,B)
// -----------------------------------------------------------------------------
//			Arithmetic between two Bitmaps
//
//     Additional parameters:
//               Arithmetic    : Arithmetic type
//               Multiplicator,
//               MaxBlendValue  : Not Used
//               Switch        : Not used
// -----------------------------------------------------------------------------
//			Blending two Bitmaps
//
//     Additional parameters:
//               Multiplicator,
//               MaxBlendValue  : The mixing of the two pictures...
//               Switch        : Not used
// -----------------------------------------------------------------------------
procedure DIBEffectArithmetic(BMPIntfc:TBMPInterface2;Arithmetic:TGraphicArithmetic;C1,C2,C3:Boolean;Multiplicator,MaxBlendValue:Integer;const EffectCallBack:TEffectCallBack);stdcall;
var
  Row, Col                :Integer;
  SrcRow1,SrcRow2,DestRow :pRGBArray;
  C1Sum,C2Sum,C3Sum     :Integer;
begin
      for Row := 0 to BMPIntfc.Height - 1 do
      begin

        if Assigned(EffectCallBack) then EffectCallBack(0,100,Round((Row/BMPIntfc.Height)*100));

        SrcRow1 := BMPIntfc.SourceScanline[Row];
        SrcRow2 := BMPIntfc.SourceScanline2[Row];
        DestRow := BMPIntfc.DestScanline[Row];
        for Col := 0 to BMPIntfc.Width - 1 do
        begin
          C1Sum:=0;C2Sum:=0;C3Sum:=0;
          CASE Arithmetic of
            gaBlend: begin
                     if C1 then C1Sum:=SrcRow1[Col].rgbtRed+xMulDiv(Multiplicator, (SrcRow2[Col].rgbtRed-SrcRow1[Col].rgbtRed), MaxBlendValue);
                     if C2 then C2Sum:=SrcRow1[Col].rgbtGreen+xMulDiv(Multiplicator, (SrcRow2[Col].rgbtGreen-SrcRow1[Col].rgbtGreen), MaxBlendValue);
                     if C3 then C3Sum:=SrcRow1[Col].rgbtBlue+xMulDiv(Multiplicator, (SrcRow2[Col].rgbtBlue-SrcRow1[Col].rgbtBlue), MaxBlendValue);
                     end;
            gaNone:begin
                     if C1 then C1Sum:=SrcRow1[Col].rgbtRed;
                     if C2 then C2Sum:=SrcRow1[Col].rgbtGreen;
                     if C3 then C3Sum:=SrcRow1[Col].rgbtBlue;
                  end;
            gaAdd:begin
                    if C1 then C1Sum:=TrimInt(0,255,SrcRow1[Col].rgbtRed+SrcRow2[Col].rgbtRed);
                    if C2 then C2Sum:=TrimInt(0,255,SrcRow1[Col].rgbtGreen+SrcRow2[Col].rgbtGreen);
                    if C3 then C3Sum:=TrimInt(0,255,SrcRow1[Col].rgbtBlue+SrcRow2[Col].rgbtBlue);
                  end;
            gaSubstract:begin
                    if C1 then C1Sum:=TrimInt(0,255,SrcRow1[Col].rgbtRed-SrcRow2[Col].rgbtRed);
                    if C2 then C2Sum:=TrimInt(0,255,SrcRow1[Col].rgbtGreen-SrcRow2[Col].rgbtGreen);
                    if C3 then C3Sum:=TrimInt(0,255,SrcRow1[Col].rgbtBlue-SrcRow2[Col].rgbtBlue);
                  end;
            gaMultiply:begin
                    if C1 then C1Sum:=TrimInt(0,255,SrcRow1[Col].rgbtRed*SrcRow2[Col].rgbtRed);
                    if C2 then C2Sum:=TrimInt(0,255,SrcRow1[Col].rgbtGreen*SrcRow2[Col].rgbtGreen);
                    if C3 then C3Sum:=TrimInt(0,255,SrcRow1[Col].rgbtBlue*SrcRow2[Col].rgbtBlue);
                  end;
            gaDivide:begin
                    if SrcRow2[Col].rgbtRed<>0 then if C1 then C1Sum:=TrimInt(0,255,SrcRow1[Col].rgbtRed DIV SrcRow2[Col].rgbtRed);
                    if SrcRow2[Col].rgbtGreen<>0 then if C2 then C2Sum:=TrimInt(0,255,SrcRow1[Col].rgbtGreen DIV SrcRow2[Col].rgbtGreen);
                    if SrcRow2[Col].rgbtBlue<>0 then if C3 then C3Sum:=TrimInt(0,255,SrcRow1[Col].rgbtBlue DIV SrcRow2[Col].rgbtBlue);
                  end;
            gaDifference:begin
                    if C1 then C1Sum:=Abs(SrcRow1[Col].rgbtRed-SrcRow2[Col].rgbtRed);
                    if C2 then C2Sum:=Abs(SrcRow1[Col].rgbtGreen-SrcRow2[Col].rgbtGreen);
                    if C3 then C3Sum:=Abs(SrcRow1[Col].rgbtBlue-SrcRow2[Col].rgbtBlue);
                  end;
            gaDarkest:begin
                    if C1 then C1Sum:=MinInt2(SrcRow1[Col].rgbtRed,SrcRow2[Col].rgbtRed);
                    if C2 then C2Sum:=MinInt2(SrcRow1[Col].rgbtGreen,SrcRow2[Col].rgbtGreen);
                    if C3 then C3Sum:=MinInt2(SrcRow1[Col].rgbtBlue,SrcRow2[Col].rgbtBlue);
                  end;
            gaLightest:begin
                    if C1 then C1Sum:=MaxInt2(SrcRow1[Col].rgbtRed,SrcRow2[Col].rgbtRed);
                    if C2 then C2Sum:=MaxInt2(SrcRow1[Col].rgbtGreen,SrcRow2[Col].rgbtGreen);
                    if C3 then C3Sum:=MaxInt2(SrcRow1[Col].rgbtBlue,SrcRow2[Col].rgbtBlue);
                  end;
            gaAverage:begin
                    if C1 then C1Sum:=TrimInt(0,255,(SrcRow1[Col].rgbtRed+SrcRow2[Col].rgbtRed)DIV 2);
                    if C2 then C2Sum:=TrimInt(0,255,(SrcRow1[Col].rgbtGreen+SrcRow2[Col].rgbtGreen)DIV 2);
                    if C3 then C3Sum:=TrimInt(0,255,(SrcRow1[Col].rgbtBlue+SrcRow2[Col].rgbtBlue)DIV 2);
                  end;
            gaBinaryOr:begin
                    if C1 then C1Sum:=TrimInt(0,255,SrcRow1[Col].rgbtRed OR SrcRow2[Col].rgbtRed);
                    if C2 then C2Sum:=TrimInt(0,255,SrcRow1[Col].rgbtGreen OR SrcRow2[Col].rgbtGreen);
                    if C3 then C3Sum:=TrimInt(0,255,SrcRow1[Col].rgbtBlue OR SrcRow2[Col].rgbtBlue);
                  end;
            gaBinaryAnd:begin
                    if C1 then C1Sum:=TrimInt(0,255,SrcRow1[Col].rgbtRed AND SrcRow2[Col].rgbtRed);
                    if C2 then C2Sum:=TrimInt(0,255,SrcRow1[Col].rgbtGreen AND SrcRow2[Col].rgbtGreen);
                    if C3 then C3Sum:=TrimInt(0,255,SrcRow1[Col].rgbtBlue AND SrcRow2[Col].rgbtBlue);
                  end;
          end;
          if not C1 then C1Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed;
          if not C2 then C2Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen;
          if not C3 then C3Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue;

          DestRow[Col].rgbtRed  := C1Sum;
          DestRow[Col].rgbtGreen  := C2Sum;
          DestRow[Col].rgbtBlue  := C3Sum;

        end;
      end;
end;


// -----------------------------------------------------------------------------
//                       *******************
//                       DIBEffect1Val
//                       *******************
// -----------------------------------------------------------------------------
//
//     Basic parameters:
//               BmpIntfc: TBMPInterface1 (One source bitmap,one destination)
//               C1,C2,C3: Color planes to be used (R,G,B)
// -----------------------------------------------------------------------------
//			Add noise
//     Additional parameters:
//               Value         : Bandwidth of noise
//               Switch        : Should noisevalue calculated for each plane
// -----------------------------------------------------------------------------
//			Anti-aliasing
//     Additional parameters:
//               Value         : Not used
//               Switch        : Not used
// -----------------------------------------------------------------------------
//			Circle around
//     Additional parameters:
//               Value         : Amount
//               Switch        : Not used
// -----------------------------------------------------------------------------
//			Contrast
//     Additional parameters:
//               Value         : Value to be added/substracted (-255..0..255)
//               Switch        : Not used
// -----------------------------------------------------------------------------
//			Ellipse
//     Additional parameters:
//               Value         : Radius (Currently not used)
//               Switch        : Not used
// -----------------------------------------------------------------------------
//			Fish-eye
//     Additional parameters:
//               Value         : Amount
//               Switch        : Not used
// -----------------------------------------------------------------------------
//			Gamma correction
//     Additional parameters:
//               Value         : Gamma offset
//               Switch        : Not used
// -----------------------------------------------------------------------------
//			Grayscale Bitmap
//     Additional parameters:
//               Value         : Not used
//               Switch        : Not used
// -----------------------------------------------------------------------------
//			Oldlook Bitmap
//     Additional parameters:
//               Value         : Not used
//               Switch        : Not used
// -----------------------------------------------------------------------------
//			Lightness
//     Additional parameters:
//               Value         : Value to be added/substracted (-255..0..255)
//               Switch        : Not used
// -----------------------------------------------------------------------------
//			Posterize Bitmap
//     Additional parameters:
//               Value         : How many Bits per channel
//               Switch        : Not used
// -----------------------------------------------------------------------------
//			Solarize
//     Additional parameters:
//               Value         : Threshold
//               Switch        : Not used
// -----------------------------------------------------------------------------
//			Spray effect
//     Parameter:
//               Value         : Intensity
//               Switch        : Not used
// -----------------------------------------------------------------------------
procedure DIBEffect1Val(BMPIntfc:TBMPInterface1;EffectType:TFXType;C1,C2,C3:Boolean;Value:Variant;Switch:Boolean;const EffectCallBack:TEffectCallBack);stdCall;
var
  TargetRow             :pRGBArray;
  SourceRow             :pRGBArray;
  nbhood                :TRGBTripleArray;  //        array[0..3] of TRGBTriple;
  C1Sum,C2Sum,C3Sum     :Integer;
  Row,Col,i,fRow,fCol,f :Integer;
  ByteArray              :Array [0..256] of Byte;
  Mask                  :Byte;
  x1, y1                :Integer;
  fx, fy, dx, dy, xmid, ymid, ar, ar2:double;

  procedure FillContrastArray;
  var j:Integer;
  begin
    for j:=0 to 126 do ByteArray[j]:=TrimInt(0,255,j-((Abs(128-j)*Value)div 256));
    for j:=127 to 255 do ByteArray[j]:=TrimInt(0,255,j+((Abs(128-j)*Value)div 256));
  end;

  procedure MakePosterizeMask;
  begin
    mask:=$FF;
    CASE Integer(Value) of
    7:Mask:=$fe;
    6:Mask:=$fc;
    5:Mask:=$f8;
    4:Mask:=$f0;
    3:Mask:=$e0;
    2:Mask:=$c0;
    1:Mask:=$80;
    else Mask:=$fe;
    end;
  end;

  function CalculateNoise(V:Integer):Integer;
  begin
    result:=Random(V)-(V shr 1);
  end;

begin
    xmid:=0;ymid:=0;ar:=0;x1:=0;y1:=0;
    if EffectType=fxAntiAlias then SetLength(nbHood,4*4);
    if EffectType=fxPosterize then MakePosterizeMask;
    if EffectType=fxContrast  then FillContrastArray;
    if EffectType=fxGamma     then for i:=0 to 255 do ByteArray[i]:=IntGamma(Double(Value),i);
    if EffectType=fxEllipse   then begin xmid := BMPIntfc.Width /2.0;ymid := BMPIntfc.Height/2.0;ar:= BMPIntfc.Height/BMPIntfc.Width;end;
    if EffectType=fxCircleAround then
    begin
      ar:=sqrt((BMPIntfc.Width-1)*(BMPIntfc.Width-1) + (BMPIntfc.Height-1)*(BMPIntfc.Height-1));
      x1:=TrimInt(0,(BMPIntfc.Width-1),Round(ar));
      y1:=TrimInt(0,(BMPIntfc.Height-1),Round(ar));
    end;

      for Row := 0 to BMPIntfc.Height - 1 do
      begin

        if Assigned(EffectCallBack) then EffectCallBack(0,100,Round((row/BMPIntfc.Height)*100));

        TargetRow:=BMPIntfc.DestScanline[Row];
        for Col := 0 to BMPIntfc.Width - 1 do
        begin
          C1Sum:=0;C2Sum:=0;C3Sum:=0;
          case EffectType of
          fxAddNoise:
            begin
              f:=CalculateNoise(Integer(Value));
              if C1 then C1Sum:=TrimInt(0,255,BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed+f);
              if (not Switch) then f:=CalculateNoise(Integer(Value));
              if C2 then C2Sum:=TrimInt(0,255,BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen+f);
              if (not Switch) then f:=CalculateNoise(Integer(Value));
              if C3 then C3Sum:=TrimInt(0,255,BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue+f);
            end;
          fxAntiAlias:
            begin
              if Col>0 then                  nbhood[0]:= BMPIntfc.SourceRGBTriple[Row,Col-1] else nbhood[0]:=BMPIntfc.SourceRGBTriple[Row,Col+1];
              if Col< BMPIntfc.Width-1 then  nbhood[1]:= BMPIntfc.SourceRGBTriple[Row,Col+1] else nbhood[1]:=BMPIntfc.SourceRGBTriple[Row,Col-1];
              if Row>0 then                  nbhood[2]:= BMPIntfc.SourceRGBTriple[Row-1,Col] else nbhood[2]:=BMPIntfc.SourceRGBTriple[Row+1,Col];
              if Row< BMPIntfc.Height-1 then nbhood[3]:= BMPIntfc.SourceRGBTriple[Row+1,Col] else nbhood[3]:=BMPIntfc.SourceRGBTriple[Row-1,Col];
              for i:=0 to 3 do
              begin
                if C1 then C1Sum:=C1Sum+nbhood[i].rgbtRed;
                if C2 then C2Sum:=C2Sum+nbhood[i].rgbtGreen;
                if C3 then C3Sum:=C3Sum+nbhood[i].rgbtBlue;
              end;
              if C1 then C1Sum:=C1Sum div 4;
              if C2 then C2Sum:=C2Sum div 4;;
              if C3 then C3Sum:=C3Sum div 4;
            end;
          fxCircleAround:
            begin
              if (Row<y1) and (Col < x1) then
              begin
                dx:=Col-BMPIntfc.Width div 2;
                dy:=Row-BMPIntfc.Height div 2;
                ar:=sqrt(dx*dx+dy*dy);
                if ar=0 then
                begin
                  fx:=0;
                  fy:=0;
                  end else
                  begin
                    if Value=0 then Value:=1;
                    ar2:=xArcTan2(dx,dy)-ar/Value - (-(Pi / 2));
                    fx:=ar* cos(ar2);
                    fy:=ar* sin(ar2);
                  end;
                  fRow:=TrimInt(0,BMPIntfc.Height-1,Trunc(fy+ (BMPIntfc.Height div 2)));
                  fCol:=TrimInt(0,BMPIntfc.Width-1,Trunc(fx+ (BMPIntfc.Width div 2)));
                  if C1 then C1Sum := BMPIntfc.SourceRGBTriple[fRow,fCol].rgbtRed;
                  if C2 then C2Sum := BMPIntfc.SourceRGBTriple[fRow,fCol].rgbtGreen;
                  if C3 then C3Sum := BMPIntfc.SourceRGBTriple[fRow,fCol].rgbtBlue;
                end;
              end;
          fxContrast:
              begin
                if C1 then C1Sum :=ByteArray[BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed];
                if C2 then C2Sum :=ByteArray[BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen];
                if C3 then C3Sum :=ByteArray[BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue];
              end;
          fxEllipse:
            begin
              ComputePixel(ar*(Col-xmid), Row-ymid, fx, fy);
	      x1 := Round(xmid+fx/ar);
	      y1 := Round(ymid+fy);
              if (y1>0) and (y1< BMPIntfc.Height-1) and
                 (x1>0) and (x1< BMPIntfc.Width-1) then
              begin
                if C1 then C1Sum:= BMPIntfc.SourceRGBTriple[y1,x1].rgbtRed;
                if C2 then C2Sum:= BMPIntfc.SourceRGBTriple[y1,x1].rgbtGreen;
                if C3 then C3Sum:= BMPIntfc.SourceRGBTriple[y1,x1].rgbtBlue;
              end
              else
              begin
                if C1 then C1Sum:= 0;
                if C2 then C2Sum:= 0;
                if C3 then C3Sum:= 0;
              end;
            end;
          fxFishEye:
            begin
              dx:=Col-(BMPIntfc.Width div 2);
              dy:=Row-(BMPIntfc.Height div 2);
              ar:=sqrt(dx*dx+dy*dy);
              if ar=0 then
              begin
                fx:=0;
                fy:=0;
              end else
              begin
                if ar=(BMPIntfc.Width * Extended(Value)) then ar:=ar+1;
                ar2:=(BMPIntfc.Width * Extended(Value)) / 2 *(1/(1-ar/(BMPIntfc.Width * Extended(Value)))-1);
                fx:=dx*ar2 /ar + (BMPIntfc.Width div 2);
                fy:=dy*ar2 /ar +(BMPIntfc.Height div 2);
              end;
              x1:=Trunc(fx);
              y1:=Trunc(fy);
              if (y1>=BMPIntfc.Height) or (x1>=bmpIntfc.Width) or (x1<0) or (y1<0) then
              begin
                if C1 then C1Sum :=0;
                if C2 then C2Sum :=0;
                if C3 then C3Sum :=0;
              end else
              begin
                   if C1 then C1Sum := BMPIntfc.SourceRGBTriple[y1,x1].rgbtRed;
                   if C2 then C2Sum := BMPIntfc.SourceRGBTriple[y1,x1].rgbtGreen;
                   if C3 then C3Sum := BMPIntfc.SourceRGBTriple[y1,x1].rgbtBlue;
              end;
            end;
          fxGamma:
            begin
                if C1 then C1Sum :=ByteArray[BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed];
                if C2 then C2Sum :=ByteArray[BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen];
                if C3 then C3Sum :=ByteArray[BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue];
            end;
          fxGrayScale:
            begin
              if C1 then C1Sum :=RgbLightness(BMPIntfc.SourceRGBTriple[Row,Col]);
              if C2 then C2Sum :=RgbLightness(BMPIntfc.SourceRGBTriple[Row,Col]);
              if C3 then C3Sum :=RgbLightness(BMPIntfc.SourceRGBTriple[Row,Col]);
            end;
          fxOldLook:
            begin
              SourceRow:=BMPIntfc.SourceScanline[Row];
              SourceRow[col].rgbtRed :=RgbLightness(BMPIntfc.SourceRGBTriple[Row,Col]);
              if C1 then C1Sum :=BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed;
              SourceRow[col].rgbtgreen :=RgbLightness(BMPIntfc.SourceRGBTriple[Row,Col]);
              if C2 then C2Sum :=BMPIntfc.SourceRGBTriple[Row,Col].rgbtgreen;
              SourceRow[col].rgbtBlue :=RgbLightness(BMPIntfc.SourceRGBTriple[Row,Col]);
              if C3 then C3Sum :=BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue;
//              if C2 then C2Sum :=RgbLightness(BMPIntfc.SourceRGBTriple[Row,Col]);
//              if C3 then C3Sum :=RgbLightness(BMPIntfc.SourceRGBTriple[Row,Col]);
            end;
          fxLightNess:
              begin
                if C1 then C1Sum :=TrimReal(0,255,(BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed)+Integer(Value));
                if C2 then C2Sum :=TrimReal(0,255,(BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen)+Integer(Value));
                if C3 then C3Sum :=TrimReal(0,255,(BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue)+Integer(Value));
              end;
          fxPosterize:
            begin
              if C2 then C2Sum :=(BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen AND Mask);
              if C1 then C1Sum :=(BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed AND Mask);
              if C3 then C3Sum :=(BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue AND Mask);
            end;
          fxSpray:
            begin
                f:=Random(Integer(Value));
                fCol:=Col+f-Random(f*2);
                fRow:=Row+f-Random(f*2);
                if(fCOl>-1)and(fCol<BmpIntfc.Width-1)and(fRow>-1)and(fRow<BMPIntfc.Height-1)then
                begin
                  if C1 then C1Sum:=BMPIntfc.SourceRGBTriple[fRow,fCol].rgbtRed;
                  if C2 then C2Sum:=BMPIntfc.SourceRGBTriple[fRow,fCol].rgbtGreen;
                  if C3 then C3Sum:=BMPIntfc.SourceRGBTriple[fRow,fCol].rgbtBlue;
                end;
            end;
          fxSolarize:
            begin
               if C1 then
               begin
                 if BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed>=Integer(Value) then C1Sum:=not BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed
                 else C1Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed;
               end;
               if C2 then
               begin
                 if BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen>=Integer(Value) then C2Sum:=not BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen
                 else C2Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen;
               end;
               if C3 then
               begin
                 if BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue>=Integer(Value) then C3Sum:=not BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue
                 else C3Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue;
               end;
             end;
           end;

          if not C1 then C1Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed;
          if not C2 then C2Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen;
          if not C3 then C3Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue;

          TargetRow[Col].rgbtRed  := C1Sum;
          TargetRow[Col].rgbtGreen  := C2Sum;
          TargetRow[Col].rgbtBlue  := C3Sum;
        end;
      end;
end;


// -----------------------------------------------------------------------------
//                       *******************
//                       DIBEffect2Val
//                       *******************
// -----------------------------------------------------------------------------
//
//     Basic parameters:
//               BmpIntfc: TBMPInterface1 (One source bitmap,one destination)
//               C1,C2,C3: Color planes to be used (R,G,B)
// -----------------------------------------------------------------------------
//			Mosaic
//     Additional parameters:
//               Value1        : Width of Mosaic
//               Value2        : Height of Mosaic
//               Switch        : Should noisevalue calculated for each plane
// -----------------------------------------------------------------------------
//			Stretch
//     Additional parameters:
//               Value1        : Lower threshold
//               Value2        : Upper threshold
//               Switch        : Not used
// -----------------------------------------------------------------------------
procedure DIBEffect2Val(BMPIntfc:TBMPInterface1;EffectType:TFXType;C1,C2,C3:Boolean;Value1,Value2:Variant;Switch:Boolean;const EffectCallBack:TEffectCallBack);stdCall;
var
  TargetRow             :pRGBArray;
  C1Sum,C2Sum,C3Sum     :Integer;
  Row,Col,x,y         :Integer;
begin
      for Row := 0 to BMPIntfc.Height - 1 do
      begin

        if Assigned(EffectCallBack) then EffectCallBack(0,100,Round((row/BMPIntfc.Height)*100));

        TargetRow:=BMPIntfc.DestScanline[Row];

        for Col := 0 to BMPIntfc.Width - 1 do
        begin
          C1Sum:=0;C2Sum:=0;C3Sum:=0;

          case EffectType of
          fxMosaic:
            begin
              x:=TrimInt(0,BMPIntfc.Width-1, ((Col div Integer(Value1)* Integer(Value1)))+(Integer(Value1) div 2));
              y:=TrimInt(0,BMPIntfc.Height-1,((Row div Integer(Value2)* Integer(Value2)))+(Integer(Value2) div 2));
              if C1 then C1Sum :=BMPIntfc.SourceRGBTriple[y,x].rgbtRed;
              if C2 then C2Sum :=BMPIntfc.SourceRGBTriple[y,x].rgbtGreen;
              if C3 then C3Sum :=BMPIntfc.SourceRGBTriple[y,x].rgbtBlue;
            end;
          fxStretch:
            begin
              if RGBIntensity(BMPIntfc.SourceRGBTriple[Row,Col])<=Integer(Value1) then
              begin
                if C1 then C1Sum:=0;
                if C2 then C2Sum:=0;
                if C3 then C3Sum:=0;
              end
              else
              if RGBIntensity(BMPIntfc.SourceRGBTriple[Row,Col])>=Integer(Value2) then
              begin
                if C1 then C1Sum:=255;
                if C2 then C2Sum:=255;
                if C3 then C3Sum:=255;
              end
              else
              begin
                if C1 then C1Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed;
                if C2 then C2Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen;
                if C3 then C3Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue;
              end;
            end;
          end;//End Case

          if not C1 then C1Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed;
          if not C2 then C2Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen;
          if not C3 then C3Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue;

          TargetRow[Col].rgbtRed  := C1Sum;
          TargetRow[Col].rgbtGreen  := C2Sum;
          TargetRow[Col].rgbtBlue  := C3Sum;
        end;// End columns
      end;//end rows
end;

// -----------------------------------------------------------------------------
//                       *******************
//                       DIBEffect3Val
//                       *******************
// -----------------------------------------------------------------------------
//
//     Basic parameters:
//               BmpIntfc: TBMPInterface1 (One source bitmap,one destination)
//               C1,C2,C3: Color planes to be used (R,G,B)
// -----------------------------------------------------------------------------
//			Colorize
//     Additional parameters:
//               Value1        : Hue increment/decrement
//               Value2        : Saturation increment/decrement
//               Value3        : Value increment/decrement
//               Switch        : Should noisevalue calculated for each plane
// -----------------------------------------------------------------------------
//			IncDecRGB
//     Additional parameters:
//               Value1        : Red increment/decrement
//               Value2        : Green increment/decrement
//               Value3        : Blue increment/decrement
//               Switch        : Not used
// -----------------------------------------------------------------------------
procedure DIBEffect3Val(BMPIntfc:TBMPInterface1;EffectType:TFXType;C1,C2,C3:Boolean;Value1,Value2,Value3:Variant;Switch:Boolean;const EffectCallBack:TEffectCallBack);stdCall;
var
  TargetRow             :pRGBArray;
  C1Sum,C2Sum,C3Sum     :Integer;
  c1b,c2b,c3b           :Byte;
  Row,Col               :Integer;
  hsl                   :THSLTriple;
begin
      for Row := 0 to BMPIntfc.Height - 1 do
      begin

        if Assigned(EffectCallBack) then EffectCallBack(0,100,Round((row/BMPIntfc.Height)*100));

        TargetRow:=BMPIntfc.DestScanline[Row];
        for Col := 0 to BMPIntfc.Width - 1 do
        begin
          C1Sum:=0;C2Sum:=0;C3Sum:=0;
          case EffectType of
          fxColorize:
            begin
              RGBtoHSL(BMPIntfc.SourceRGBTriple[Row,Col],hSl);
              hsl.hslHue:=hsl.hslHue+Integer(Value1);
              hsl.hslSaturation:=hsl.hslSaturation+Integer(Value2);
              hsl.hslLightness:=hsl.hslLightness+Integer(Value3);
              hsl.hslHue:=LoopInt(0,359,hsl.hslHue);
              hsl.hslSaturation:=TrimInt(0,255,hsl.hslSaturation);
              hsl.hslLightness:=TrimInt(0,255,hsl.hslLightness);
              HSLToRGB2(hsl,c1b,c2b,c3b);
              if C1 then C1Sum:=Integer(c1b);
              if C2 then C2Sum:=Integer(c2b);
              if C3 then C3Sum:=Integer(c3b);
            end;
          fxIncDecRGB:
            begin
              if C1 then C1Sum:=TrimReal(0,255,(BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed)+Integer(Value1));
              if C2 then C2Sum:=TrimReal(0,255,(BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen)+Integer(Value2));
              if C3 then C3Sum:=TrimReal(0,255,(BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue)+Integer(Value3));
            end;
          end;
          if not C1 then C1Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtRed;
          if not C2 then C2Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtGreen;
          if not C3 then C3Sum:=BMPIntfc.SourceRGBTriple[Row,Col].rgbtBlue;

          TargetRow[Col].rgbtRed  := C1Sum;
          TargetRow[Col].rgbtGreen  := C2Sum;
          TargetRow[Col].rgbtBlue  := C3Sum;
        end;
      end;
end;

// Helper functions and procedures


function ComputePixel(x, y:double;var x1,y1:double):Integer; //float &x1, float &y1)
var
  r, nn : double;
begin
  if (x=0) and (y=0) then
  begin
   x1 := x;
   y1 := y;
   result:=1;
  end
  else
  begin
    nn := sqrt(x*x*0.5 + y*y);
    if (abs(x) > abs(y)) then r:=abs(nn/x) else r:=abs(nn/y);
    x1 := (r*x);
    y1 := (r*y);
    result:= 1;
  end;
end;

end.
